export class Main extends Indicator {

    constructor() {
        super();

        this.setProperty(Property.SeparateWindow, true);
        this.setProperty(Property.Minimum, 0);

        this.addInput("ExtStdDevPeriod", InputType.int, 20);
        this.addInput("ExtStdDevMAMethod", InputType.int, 0);
        this.addInput("ExtStdDevAppliedPrice", InputType.AppliedPrice, AppliedPrice.Close);
        this.addInput("ExtStdDevShift", InputType.int, 0);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            ExtStdDevBuffer : this.addBuffer(),
        };
    }

    onInit() {
        let {ExtStdDevPeriod, ExtStdDevShift} = this.getInputs();

        this.buffers.ExtStdDevBuffer
            .setShape(Shape.Line)
            .setLabel("StdDev(" + ExtStdDevPeriod + ")")
            .setDrawBegin(ExtStdDevPeriod)
            .setShortName("StdDev(" + ExtStdDevPeriod + ")")
            .setShift(ExtStdDevShift)
            .setColor(Color.Blue)
        ;
    }

    async onUpdate() {
        const  {ExtStdDevPeriod, ExtStdDevMAMethod, ExtStdDevAppliedPrice, PriceType} = this.getInputs(),
            {ExtStdDevBuffer} = this.buffers,
            barSize = await Bar.size();

        let i, j, countedBars, dAPrice, dAmount, dMovingAverage, list, appliedPrices;

        if (barSize <= ExtStdDevPeriod) return;

        countedBars = Bar.counted();
        i = barSize - ExtStdDevPeriod - 1;
        if (countedBars > ExtStdDevPeriod) {
            i = barSize - countedBars;
        }

        list = await Indicators.MA(null, 0, ExtStdDevPeriod, 0, ExtStdDevMAMethod, ExtStdDevAppliedPrice, PriceType);
        appliedPrices = await Bar.appliedPrice(ExtStdDevAppliedPrice, PriceType);
        while (i >= 0) {
            dAmount = 0;
            dMovingAverage = list[i] || 0;
            for (j = 0; j < ExtStdDevPeriod; j++) {
                dAPrice = appliedPrices[(i + j)];
                dAmount += (dAPrice - dMovingAverage) * (dAPrice - dMovingAverage);
            }
            ExtStdDevBuffer.set(i, (Math.sqrt(dAmount / ExtStdDevPeriod)));
            i--;
        }


    }
}